# - Copyright UKFast.net Limited 2020
# - Contact: defensive.security@ukfast.co.uk
#
# - Link Nessus agent
# - Version: 1.0.0
# - Author: JR 13/10/2020
# - This script is to be used by UKFast Threat Vision

# This script is used to update link a Nessus agent to Threat Scan
# Usage ./SCRIPT [OPTIONS]
# -help - Show this help page
# -debug | Run script in debug mode
# -nessus_host | nessus manager 
# -port | port of nessus manager
# -name | The Nessus agent name
# -group | The nessus group
# -key | The nessus linking key

param(
    [switch]$debug,
    [switch]$help,
    [string]$name,
    [string]$nessus_host,
    [string]$port,
    [string]$group,
    [string]$key
)

$logfile = "C:\Program Files (x86)\ossec-agent\active-response\active-responses.log"

function JSONARLog {
    param (
        [string]$message,
        [boolean]$bool_success
    )
    "{`"date_time`":`"$(date)`",`"type`":`"active_response_log`",`"success`":`"$bool_success`",`"script_path`":`"/`",`"script_name`":`"nessus_agent_link`",`"status_message`":`"$message`"}" | Out-File -FilePath $logfile -encoding ASCII -Append
}

function checkDependencies {
    # Check if the nessuscli file exists
    If (Test-Path -Path "c:\Program Files\Tenable\Nessus Agent\nessuscli.exe") {
        If ($debug -eq $true){
            JSONARlog -bool_success $true -message "[Info] Detected dependency nessuscli file"
        }
    }
    Else {
        JSONARlog -bool_success $false -message "[Error] Dependency nessuscli file not found."
        exit
    }
}

function check_cli_arguments {
    param (
        [string]$name,
        [string]$nessus_host,
        [string]$port,
        [string]$group,
        [string]$key
    )
    If ($key){
        If ($debug -eq $true){
            JSONARlog -bool_success $true -message "[Info] NESSUS_KEY argument has been set is $key"
        }
    }
    Else {
        If ($debug -eq $true){
            JSONARlog -bool_success $false -message "[Error] NESSUS_KEY argument has not been set"
        }
        exit
    }
    If ($nessus_host){
        If ($debug -eq $true){
            JSONARlog -bool_success $true -message "[Info] NESSUS_HOST argument has been set is $nessus_host"
        }
    }
    Else{
        If ($debug -eq $true){
            JSONARlog -bool_success $false -message "[Error] NESSUS_HOST argument has not been set"
        }
        exit
    }
    If ($port){
        If ($debug -eq $true){
            JSONARlog -bool_success $true -message "[Info] NESSUS_PORT argument has been set is $port"
        }
    }
    Else{
        If ($debug -eq $true){
            JSONARlog -bool_success $false -message "[Error] NESSUS_PORT argument has not been set"        
        }
        exit
    }
    If ($name){
        If ($debug -eq $true){
            JSONARlog -bool_success $true -message "[Info] AGENT_NAME argument has been set is $name"
        }
    }
    Else{
        If ($debug -eq $true){
            JSONARlog -bool_success $false -message "[Error] AGENT_NAME argument has not been set"       
        }
        exit
    }
    If ($group){
        If ($debug -eq $true){
            JSONARlog -bool_success $true -message "[Info] AGENT_GROUP argument has been set is $group"
        }
    }
    Else{
        If ($debug -eq $true){
            JSONARlog -bool_success $false -message "[Error] AGENT_GROUP argument has not been set"       
        }
        exit
    }
}

function link_nessus {
    param (
        [string]$name,
        [string]$nessus_host,
        [string]$port,
        [string]$group,
        [string]$key
    )
    #Invoke-Expression "'C:\Program Files\Tenable\Nessus Agent\nessuscli.exe' agent link --key=$key --name=$name --host=$nessus_host --port=$port --groups=$group" -OutVariable nessus_output -ErrorVariable nessus_error_output
    $nessus_output = (& 'C:\Program Files\Tenable\Nessus Agent\nessuscli.exe' agent link --key=$key --name=$name --host=$nessus_host --port=$port --groups=$group) 2>&1
    If ($nessus_output -like '*already exists*'){
        JSONARlog -bool_success $true -message "[Success] Nessus agent $name is already linked."
    }
    ElseIf ($nessus_output -like '*Successfully linked to*') {
        JSONARlog -bool_success $true -message "[Success] Nessus agent $name was successfully linked."     
    }
    Else {
        JSONARlog -bool_success $false -message "[Error] Error while running link command for $name"
        $nessus_output = $nessus_output -replace "`n",", " -replace "`r",", "
        JSONARlog -bool_success $false -message "$nessus_output"
        exit
    }
}

checkDependencies
check_cli_arguments -name $name -nessus_host $nessus_host -port $port -group $group -key $key
link_nessus -name $name -nessus_host $nessus_host -port $port -group $group -key $key   
#JSONARlog -bool_success $true -message "[Success] All steps completed"  
